==> viewer2d-0.2.4.tgz: package/dev/texter.js <==
/**
 * @param { Viewer } viewer
 */
function addTexter(viewer, type = "IfcSpace") {
  const texts = new Map();
  let textsArray = null;
  const setTextsArray = () => {
    textsArray = Array.from(texts.values());
  };

  const texterElement = document.getElementById("texter");

  let id = 1;

  const texter = {
    addText(text, objectId, priority = 0) {
      // reset cache
      textsArray = null;

      const object = viewer.scene.objectsMap.get(objectId);
      if (object) {
        const { position, textEl } = addTextElement(texterElement, text);

        const textObject = {
          id: id++,
          position,
          object,
          text,
          textEl,
          get rect() {
            return this.textEl.getBoundingClientRect();
          },
          get visible() {
            return textObject.textEl.style.color !== "transparent";
          },
          set visible(value) {
            if (value) {
              textObject.textEl.style.color = null;
            } else {
              textObject.textEl.style.color = "transparent";
            }
          },
          collidesWith: new Set(),
          priority,
          destroy() {
            const div = this.textEl.parentElement;
            div.parentElement.removeChild(div);

            texts.delete(this.object.id);
          },
        };

        texts.set(object.id, textObject);

        updatePosition(textObject, viewer.camera.transform);

        updateTexts(viewer.camera.transform);

        return text;
      } else {
        console.warn(
          `Cannot add text to object with id ${objectId} because it does not exist`
        );

        return false;
      }
    },
    removeText(objectId) {
      const textObject = texts.get(objectId);
      if (textObject) {
        textObject.destroy();
        return true;
      } else {
        return false;
      }
    },
    clear() {
      texts.forEach(textObject => textObject.destroy());
    },
  };

  const updatePosition = (textObject, transform) => {
    const { x: pointX, y: pointY } = textObject.object.geometry.center;

    const { x = 0, y = 0 } = transform.apply({
      x: pointX,
      y: pointY,
    });

    const position = textObject.position;
    position.x = x / window.devicePixelRatio;
    position.y = y / window.devicePixelRatio;
  };

  const updateTexts = transform => {
    texts.forEach(textObject => updatePosition(textObject, transform));
    // cache textArray
    if (!textsArray) {
      setTextsArray();
    }
    textsArray.forEach(updateCollisions);
    textsArray.forEach(updateVisibility);
  };

  const updateCollisions = (textObject, i) => {
    textsArray.slice(i + 1).forEach(textObjectToCompare => {
      if (isIntersecting(textObjectToCompare.rect, textObject.rect)) {
        textObjectToCompare.collidesWith.add(textObject);
        textObject.collidesWith.add(textObjectToCompare);
      } else {
        if (textObjectToCompare.collidesWith.has(textObject)) {
          textObjectToCompare.collidesWith.delete(textObject);
        }
        if (textObject.collidesWith.has(textObjectToCompare)) {
          textObject.collidesWith.delete(textObjectToCompare);
        }
      }
    });
  };

  viewer.camera.on("update", updateTexts);

  viewer.scene.on("object-update", ({ object, property }) => {
    const textObject = texts.get(object.id);
    if (textObject && property === "visible" && object.visible === false) {
      textObject.destroy();
    }
  });

  viewer.scene.on("object-removed", object => {
    const textObject = texts.get(object.id);
    if (textObject) {
      textObject.destroy();
    }
  });

  const button = document.getElementById("addTexts");
  let textsDisplayed = false;
  button.onclick = () => {
    if (textsDisplayed) {
      texter.clear();
    } else {
      const objectsToText = viewer.scene.objects.filter(
        object => object.visible && object.type === type
      );
      objectsToText.forEach(object => {
        texter.addText(object.type, object.id, object.area);
      });
    }
    textsDisplayed = !textsDisplayed;
  };
}

function updateVisibility(textObject) {
  if (textObject.visible) {
    if (
      textObject.collidesWith.size > 0 &&
      Array.from(textObject.collidesWith.values()).some(
        colliding =>
          colliding.visible &&
          (colliding.priority > textObject.priority ||
            (colliding.priority === textObject.priority &&
              colliding.id > textObject.id))
      )
    ) {
      textObject.visible = false;
    }
  } else {
    if (textObject.collidesWith.size > 0) {
      if (
        !Array.from(textObject.collidesWith.values()).some(
          colliding =>
            colliding.visible &&
            (colliding.priority > textObject.priority ||
              (colliding.priority === textObject.priority &&
                colliding.id > textObject.id))
        )
      ) {
        textObject.visible = true;
      }
    } else {
      textObject.visible = true;
    }
  }
}

/**
 * @param { DOMRect } rect1
 * @param { DOMRect } rect2
 */
function isIntersecting(rect1, rect2) {
  return (
    rect1.bottom > rect2.top &&
    rect1.right > rect2.left &&
    rect1.top < rect2.bottom &&
    rect1.left < rect2.right
  );
}

function addTextElement(parent, text) {
  // create a div with a center according to the text length
  const centeredDiv = document.createElement("div");
  centeredDiv.style.display = "inline-block";
  centeredDiv.style.transform = "translate(-50%, -50%)";
  centeredDiv.style.position = "absolute";
  centeredDiv.style.pointerEvents = "none";
  centeredDiv.style.color = "white";

  const textContainer = document.createElement("span");
  textContainer.style.display = "inline-block";
  // textContainer.style.fontSize = "x-large";
  textContainer.style.whiteSpace = "nowrap";
  // textContainer.style.border = "solid blue 1px"; // dev only
  textContainer.style.lineHeight = "20px"; // TODO may not work in all cases.

  const textNode = document.createTextNode(text);
  textContainer.appendChild(textNode);

  const position = new Proxy(Object.seal({ x: 0, y: 0 }), {
    set(target, property, value, receiver) {
      const res = Reflect.set(target, property, value, receiver);
      textContainer.style.transform = `translate(${target.x}px, ${target.y}px)`;
      return res;
    },
  });

  centeredDiv.appendChild(textContainer);
  parent.appendChild(centeredDiv);

  return { position, textEl: textContainer };
}

export default addTexter;

==> viewer2d-0.3.1.tgz: package/dev/texter.js <==
/**
 * @param { Viewer } viewer
 */
function addTexter(viewer, type = "IfcSpace") {
  const texts = new Map();
  let textsArray = null;
  const setTextsArray = () => {
    textsArray = Array.from(texts.values());
  };

  const texterElement = document.getElementById("texter");

  let id = 1;

  const texter = {
    addText(text, objectId, priority = 0) {
      // reset cache
      textsArray = null;

      const object = viewer.scene.objectsMap.get(objectId);
      if (object) {
        const { position, textEl, textNode } = addTextElement(
          texterElement,
          text
        );

        const textObject = {
          id: id++,
          position,
          object,
          get text() {
            return textNode.nodeValue;
          },
          set text(value) {
            textNode.nodeValue = value;
          },
          textEl,
          get rect() {
            return this.textEl.getBoundingClientRect();
          },
          get visible() {
            return textObject.textEl.style.color !== "transparent";
          },
          set visible(value) {
            if (value) {
              textObject.textEl.style.color = null;
            } else {
              textObject.textEl.style.color = "transparent";
            }
          },
          collidesWith: new Set(),
          priority,
          destroy() {
            const div = this.textEl.parentElement;
            div.parentElement.removeChild(div);

            texts.delete(this.object.id);
          },
        };

        texts.set(object.id, textObject);

        updatePosition(textObject, viewer.camera.transform);

        updateTexts(viewer.camera.transform);

        return textObject;
      } else {
        console.warn(
          `Cannot add text to object with id ${objectId} because it does not exist`
        );

        return undefined;
      }
    },
    removeText(objectId) {
      const textObject = texts.get(objectId);
      if (textObject) {
        textObject.destroy();
        return true;
      } else {
        return false;
      }
    },
    clear() {
      texts.forEach(textObject => textObject.destroy());
    },
  };

  const updatePosition = (textObject, transform) => {
    const { x: pointX, y: pointY } = textObject.object.geometry.center;

    const { x = 0, y = 0 } = transform.apply({
      x: pointX,
      y: pointY,
    });

    const position = textObject.position;
    position.x = x / window.devicePixelRatio;
    position.y = y / window.devicePixelRatio;
  };

  const updateTexts = transform => {
    texts.forEach(textObject => updatePosition(textObject, transform));
    // cache textArray
    if (!textsArray) {
      setTextsArray();
    }
    textsArray.forEach(updateCollisions);
    textsArray.forEach(updateVisibility);
  };

  const updateCollisions = (textObject, i) => {
    textsArray.slice(i + 1).forEach(textObjectToCompare => {
      if (isIntersecting(textObjectToCompare.rect, textObject.rect)) {
        textObjectToCompare.collidesWith.add(textObject);
        textObject.collidesWith.add(textObjectToCompare);
      } else {
        if (textObjectToCompare.collidesWith.has(textObject)) {
          textObjectToCompare.collidesWith.delete(textObject);
        }
        if (textObject.collidesWith.has(textObjectToCompare)) {
          textObject.collidesWith.delete(textObjectToCompare);
        }
      }
    });
  };

  viewer.camera.on("update", updateTexts);

  viewer.scene.on("object-update", ({ object, property }) => {
    const textObject = texts.get(object.id);
    if (textObject && property === "visible" && object.visible === false) {
      textObject.destroy();
    }
  });

  viewer.scene.on("object-removed", object => {
    const textObject = texts.get(object.id);
    if (textObject) {
      textObject.destroy();
    }
  });

  const button = document.getElementById("addTexts");
  let textsDisplayed = false;
  button.onclick = () => {
    if (textsDisplayed) {
      texter.clear();
    } else {
      const objectsToText = viewer.scene.objects.filter(
        object => object.visible && object.type === type
      );
      objectsToText.forEach(object => {
        texter.addText(object.type, object.id, object.area);
      });
    }
    textsDisplayed = !textsDisplayed;
  };

  return texter;
}

function updateVisibility(textObject) {
  if (textObject.visible) {
    if (
      textObject.collidesWith.size > 0 &&
      Array.from(textObject.collidesWith.values()).some(
        colliding =>
          colliding.visible &&
          (colliding.priority > textObject.priority ||
            (colliding.priority === textObject.priority &&
              colliding.id > textObject.id))
      )
    ) {
      textObject.visible = false;
    }
  } else {
    if (textObject.collidesWith.size > 0) {
      if (
        !Array.from(textObject.collidesWith.values()).some(
          colliding =>
            colliding.visible &&
            (colliding.priority > textObject.priority ||
              (colliding.priority === textObject.priority &&
                colliding.id > textObject.id))
        )
      ) {
        textObject.visible = true;
      }
    } else {
      textObject.visible = true;
    }
  }
}

/**
 * @param { DOMRect } rect1
 * @param { DOMRect } rect2
 */
function isIntersecting(rect1, rect2) {
  return (
    rect1.bottom > rect2.top &&
    rect1.right > rect2.left &&
    rect1.top < rect2.bottom &&
    rect1.left < rect2.right
  );
}

function addTextElement(parent, text) {
  // create a div with a center according to the text length
  const centeredDiv = document.createElement("div");
  centeredDiv.style.display = "inline-block";
  centeredDiv.style.transform = "translate(-50%, -50%)";
  centeredDiv.style.position = "absolute";
  centeredDiv.style.pointerEvents = "none";
  centeredDiv.style.color = "white";

  const textContainer = document.createElement("span");
  textContainer.style.display = "inline-block";
  // textContainer.style.fontSize = "x-large";
  textContainer.style.whiteSpace = "nowrap";
  // textContainer.style.border = "solid blue 1px"; // dev only
  textContainer.style.lineHeight = "20px"; // TODO may not work in all cases.

  const textNode = document.createTextNode(text);
  textContainer.appendChild(textNode);

  const position = new Proxy(Object.seal({ x: 0, y: 0 }), {
    set(target, property, value, receiver) {
      const res = Reflect.set(target, property, value, receiver);
      textContainer.style.transform = `translate(${target.x}px, ${target.y}px)`;
      return res;
    },
  });

  centeredDiv.appendChild(textContainer);
  parent.appendChild(centeredDiv);

  return { position, textEl: textContainer, textNode };
}

export default addTexter;
